#include "MAX5715.h"

#include "hal_data.h"
#include "r_spi.h"
#include "SEGGER_RTT/SEGGER_RTT.h"
#include "FreeRTOS.h"
#include "task.h"

#include <stdint.h>

void MAX5715_SpiCb(spi_callback_args_t* args);
extern spi_instance_ctrl_t g_spi0_ctrl;
extern const spi_cfg_t g_spi0_cfg;

static int completed = 0;

void MAX5715_Init() {
	fsp_err_t fStatus;

	fStatus = R_SPI_Open(&g_spi0_ctrl, &g_spi0_cfg);
	if (fStatus) {
		SEGGER_RTT_printf(0, "MAX5715 R_SPI_Open Failed.\r\n");
	}

}


void MAX5715_SpiCb(spi_callback_args_t* args) {
	completed = 1;
	SEGGER_RTT_printf(0, "MAX5715 CB event=%d\r\n", args->event);
}

static void MAX5715_Transmit(uint32_t message) {
	fsp_err_t fStatus;

	uint8_t buffer[3];
	buffer[2] = (message & 0xFF0000) >> 16;
	buffer[1] = (message & 0xFF00) >> 8;
	buffer[0] = (message & 0xFF);

	fStatus = R_SPI_CallbackSet(&g_spi0_ctrl, MAX5715_SpiCb, NULL, NULL);
	if (fStatus) {
		SEGGER_RTT_printf(0, "MAX5715 R_SPI_CallbackSet Failed.\r\n");
	}

	completed = 0;

	fStatus = R_SPI_Write(&g_spi0_ctrl, buffer, 1, SPI_BIT_WIDTH_24_BITS);
	if (fStatus) {
		SEGGER_RTT_printf(0, "RMAX5715 _SPI_Write Failed.\r\n");
	}

	while (completed == 0) {
		vTaskDelay(1);
	}
}

void MAX5715_SetReference(int referencePowerAllways, MAX5715_Reference mode) {
	MAX5715_Transmit(0x700000 | ((!!referencePowerAllways) << 18) | ((mode & 3) << 16));
}

void MAX5715_SetDAC(MAX5715_Channel channels, uint16_t value) {
	MAX5715_Transmit(0x200000 | ((channels & 0xF) << 16) | ((value & 0xFFF) << 4));
}
