#include "dac_thrd.h"
#include "MAX5715.h"

#include "SEGGER_RTT/SEGGER_RTT.h"

MAX5715_Channel motorDacChannel[3] = {MAX5715_Channel_C, MAX5715_Channel_B, MAX5715_Channel_A};
uint16_t targetDacValue[3] = {0, 0, 0};
uint16_t actualDacValue[3] = {0xFFFF, 0xFFFF, 0xFFFF};

void MotorCurrentLimit_SetInternalLimitter(int motorNumber) {
	targetDacValue[motorNumber] = 0;
}

void MotorCurrentLimit_SetLimit(int motorNumber, float current) {
	if (current < 0.45) {
		current = 0.45;
	}
	if (current > 2) { // limited by DAC reference
		current = 2;
	}

	uint16_t dacVal = (uint16_t)(4095 * current / 2.048);

	targetDacValue[motorNumber] = dacVal;
}


void dac_thrd_entry(void *pvParameters) {
	FSP_PARAMETER_NOT_USED(pvParameters);

	MAX5715_Init();
	MAX5715_SetDAC(MAX5715_Channel_All, 0); // Set default Internal current limit of MAX14871
	MAX5715_SetReference(1, MAX5715_Reference_2V048); // Max limit 2.048 A

	while (1) {
		for (int i = 0; i < 3; i++) {
			uint16_t target = targetDacValue[i];
			if (target != actualDacValue[i]) {
				MAX5715_SetDAC(motorDacChannel[i], target);
				actualDacValue[i] = target;
			}
		}

		vTaskDelay(1);
	}
}
